Package: fmcmc
Title: A friendly MCMC framework
Version: 0.5-2
Date: 2023-08-29
Authors@R: c(person("George", "Vega Yon", email = "g.vegayon@gmail.com", role =
         c("aut", "cre"), comment = c(ORCID = "0000-0002-3171-0844")),
         person("Paul", "Marjoram", email = "pmarjora@usc.edu", role = c("ctb",
         "ths"), comment = c(ORCID = "0000-0003-0824-7449")), person("National
         Cancer Institute (NCI)", role = "fnd", comment = "Grant Number
         5P01CA196569-02"), person("Fabian", "Scheipl", role = "rev", comment =
         c(what = "JOSS reviewer", ORCID="0000-0001-8172-3603")) )
Description: Provides a friendly (flexible) Markov Chain Monte Carlo (MCMC)
         framework for implementing Metropolis-Hastings algorithm in a modular way
         allowing users to specify automatic convergence checker, personalized
         transition kernels, and out-of-the-box multiple MCMC chains using
         parallel computing. Most of the methods implemented in this package can
         be found in Brooks et al. (2011, ISBN 9781420079425). Among the methods
         included, we have: Haario (2001) <doi:10.1007/s11222-011-9269-5>
         Adaptive Metropolis, Vihola (2012) <doi:10.1007/s11222-011-9269-5>
         Robust Adaptive Metropolis, and Thawornwattana et
         al. (2018) <doi:10.1214/17-BA1084> Mirror transition kernels.
Depends: R (>= 3.3.0)
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
LazyData: true
URL: https://github.com/USCbiostats/fmcmc
BugReports: https://github.com/USCbiostats/fmcmc/issues
Suggests: covr, knitr, rmarkdown, mcmc, tinytest, mvtnorm,
Imports: parallel, coda, stats, methods, MASS, Matrix
RoxygenNote: 7.2.3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2023-08-29 23:20:58 UTC; george
Author: George Vega Yon [aut, cre] (<https://orcid.org/0000-0002-3171-0844>),
  Paul Marjoram [ctb, ths] (<https://orcid.org/0000-0003-0824-7449>),
  National Cancer Institute (NCI) [fnd] (Grant Number 5P01CA196569-02),
  Fabian Scheipl [rev] (JOSS reviewer,
    <https://orcid.org/0000-0001-8172-3603>)
Maintainer: George Vega Yon <g.vegayon@gmail.com>
Repository: CRAN
Date/Publication: 2023-08-29 23:50:02 UTC
Built: R 4.6.0; ; 2026-04-27 06:40:31 UTC; unix
